function [topper, lotter, rotter] = mixedup(w, left, right)
% [PEAKS, LEFTMIN, RIGHTMIN] = mixedup(spectrum, smallWindow, largeWindow)
%
% Input consists of a spectrum and two window sizes. The smallWindow is
% used to collapse potential peaks together at the left end of the
% spectrum; ther default size is 5 ticks. The largeWindow is used to
% collapse peaks together at the right end of the spectrum; the default
% size is 30. The window sizes are smoothly interpolated along a
% quadratic curve, which makes them a fixed percentage of the mass on
% that scale.
%
% The return values are:
%	PEAKS: a list of peak locations, in ticks
%	LEFTMIN:	a list of the closest minimum to the left of each peak
%	RIGHTMIN: a list of the closest minimum to the right of each peak
%
% Described as algorithm SPF in Clinical Chemistry 2003; 49:1615-1623.
%
% Original version by Kevin R. Coombes
% $Revision: 2.2 $
% Last modified by $Author: krc $ on $Date: 2003/03/31 19:01:58 $.

% set up default values for optional arguments
if nargin < 3,
   right = 30;
end
if nargin < 2,
   left = 5;
end

% perform a coherency check in case someone gave us nonsensical values
if left <= 0,
   left = 5
end
if right < left,
   temp = right;
   right = left;
   left = temp;
end


%% Step 1: Find all local maxima and associated local minima.
x = diff(w);		% first difference
xl = x(1:end-1);	% left side
xr = x(2:end);		% right side
neartop = (xl > 0 & xr <= 0) | (xl == 0 & xr < 0);
nearbot = (xl < 0 & xr >= 0) | (xl == 0 & xr > 0);
xx = 1:length(x);
nb = [1 xx(nearbot)+1];	% tentative minimum
nt = xx(neartop)+1;		% tentative maximum

%% Step 2: Coalesce rising plateaus to find the right-most one.
curt = 1;
curb = 1;
tops = [0];
bots = [1];
numtop = 1;
numbot = 1;
while curt < length(nt) & curb < length(nb),
	while nb(curb)<nt(curt) & curb < length(nb),
   	curb = curb+1;
	end
	bots(numbot) = nb(curb);
	numbot = numbot+1;
	while nt(curt) < nb(curb) & curt < length(nt),
	   curt = curt + 1;
	end
	tops(numtop) = nt(curt);
   numtop = numtop + 1;
end
clear numtop numbot curt curb nb nt nearbot neartop

% At this point, "tops" and "bots" contain local maxima and minima
% including those that go up for one tick and then immediately go
% back down. The arrays should be the same size, with each minimum
% occuring strictly to the left of each maximum. In a test example,
% there were about 3500 of these local maxima in a spectrum that
% contained 18000 points.

%% Step 3: Cull some obviously inadequate candidates. Anything where the
% distance from minimum to maximum is less than the median single step
% size gets removed. In our example, this step got us down to about
% 1500 potential peaks.
fuzz = median(abs(x));		% typical first difference; a noise estimate
flux = w(tops) - w(bots);	% size of the step up from bottom to top
tops = tops(flux > 2*fuzz);
bots = bots(flux > 2*fuzz);
clear flux

%% Step 4: Further culling. We combine maxima that are obviously too
% close together to be resolved. The resolution is determined by
% the arguments specifying window sizes. In our test example, this
% step got us down from about 1500 peaks to about 500.

dt = diff(tops);	% distance in ticks between successive tops
a = (right-left)./length(x)^2;		% define "close" using y = a*x^2 + b
resolve = left + (a.*tops).*tops;	% using parameters left and right.

temtops = tops;
topper = [1];	% list of resolvable maxima.
i = 1;
while i < length(temtops)-1,
   if dt(i) < resolve(i),
      if w(temtops(i)) < w(temtops(i+1)),
         temtops(i) = temtops(i+1);
      else
         temtops(i+1) = temtops(i);
      end
      dt(i+1) = temtops(i+2)-temtops(i);
   else
      topper(end+1) = temtops(i);
   end
   i = i + 1;
end
clear dt a resolve

% After combining maxima, we then figure out where the closest minima
% are to both left and right. These might not agree for adjacent peaks
% because of the presence of plateaus.
lotter = [1];	% list of resolvable minima to left of peak.
rotter = [1];	% similar list to right of peak.
for i = 2:length(topper),
   temp=topper(i-1):topper(i);
   lotter(i) = max(temp(w(temp)==min(w(temp))));
   rotter(i) = min(temp(w(temp)==min(w(temp))));
end
rotter = [rotter(2:end) length(x)];

% Step 5: The primary goal of this step is to strip out candidate
% peaks that lie on the slope up to a true peak. We insist that every
% peak include a minimum number of "up ticks" and "down ticks" from
% the maximum to the neighboring minima.

uptick = topper-lotter;
upper = median(uptick)-median(abs(uptick-median(uptick)));
dntick = rotter-topper;
downer = median(dntick)-median(abs(dntick-median(dntick)));
magic = min(upper, downer) + 1;
keeper = ~ (uptick < magic | dntick < magic);

topper = topper(keeper);
rotter = rotter(keeper);
lotter = lotter(keeper);

%% Step 6: Refine the peak location. We may have culled the wrong
% peak when two peaks were close together. Thus, we again locate the
% local maximum between two successive minima.

bottle = [lotter length(x)];
for i = 1:length(topper),
   temp=bottle(i):bottle(i+1);
   topper(i) = max(temp(w(temp)==max(w(temp))));
end
clear bottle temp

% Of course, if the peaks move, we may need to adjust the bases as well.
lotter = [1];	% list of resolvable minima to left of peak.
rotter = [1];	% similar list to right of peak.
for i = 2:length(topper),
   temp=topper(i-1):topper(i);
   lotter(i) = max(temp(w(temp)==min(w(temp))));
   rotter(i) = min(temp(w(temp)==min(w(temp))));
end
maxright = min(topper(end)+100, length(w));
rotter = [rotter(2:end) maxright];






