function [peaks, newspec, sn, y3, midspec, y, y2] = peakDetection(spectrum, snminimum, left, right, lwin, rwin, mono, multiplier)
% [PEAKS, NEWSPEC] = peakDetection(spectrum, sn, L, R, LB, RB, mono, multiplier)
%
% Given a spectrum, we want to remove the baseline and identify the
% location of peaks. The main return from this function is a list,
% PEAKS, of the peak locations in ticks. The second return value is
% the modified spectrum with the baseline removed.
%
% The second argument, snminimum, is the minimum acceptable value for
% the signal to noise ratio. The next pair of values are the left and
% right window sizes for peak resolution; see MIXEDUP for details. The
% next pair of values are the left and right window sizes for peak
% removal in preparation for baseline correction; see LOCALMINPET for
% details. The next value specifies whether the baseline should be made
% monotonically decreasing. The final value is a multiplier specifying
% the number of peak widths used to calculate the noise; this defaults to 5.
%
% Described as algorithm SPDBC in Clinical Chemistry 2003; 49:1615-1623.
%
% Original version by Kevin R. Coombes
% $Revision: 2.2 $
% Last modified by $Author: krc $ on $Date: 2003/03/31 19:01:58 $.

if nargin < 8,
   multiplier = 5;
end


% step 1: Use our basic peak finder on the raw spectrum.
[y, yleft, yright] = mixedup(spectrum, left, right);

% step 2: Remove the peaks and fill in with straight lines
% across the base.
slip = peakRemoval(spectrum, y, yleft, yright, mono);

% step 3: Compute a local minimum in a window of width 256. (The
% argument to the function is 128, which extends on each side from
% the center.)
mini1 = localMinPet(slip, lwin, rwin, mono);
mini1 = reshape(mini1, size(spectrum));	% residual nonsense

% step 4: Remove the local minimum to estimate the base line.
midspec = spectrum - mini1;

% step 5: Repeat steps one through four. The first pass typically
% leaves a slight slant to the baseline in the exponentially decaying
% region, and a second pass cleans up most of this.
[y2, yleft2, yright2] = mixedup(midspec, left, right);
slip2 = peakRemoval(midspec, y2, yleft2, yright2, mono);
mini = localMinPet(slip2, lwin, rwin, mono);
mini = reshape(mini, size(spectrum));	
newspec = midspec - mini;

% step 6: Make a final peak finding pass on the baseline-corrected
% spectrum.
[y3, yleft3, yright3] = mixedup(newspec, left, right);

% step 7: In general, the peak finder is too inclusive, since it does
% not take noise into consideration. As described in that file, the
% general process found about 450 peaks in a spectrum of 18000 points.
% Visual inspection revealed that many of them were still in the noise
% region. So, we compute a signal to noise ratio using the MAD in a
% local window, and only keep peaks above a threshold. Using S/N > 3,
% we get down to about 200 peaks, most of which seem believeable.

width = multiplier*max(yright3-y3, y3-yleft3);

noise = 0*y3;
middle = 0*y3;
for i = 1:length(y3),
	lend = max(1, y3(i)-width(i));
   rend = min(length(newspec), y3(i)+width(i));
   slice = [newspec(lend:yleft3(i)) newspec(yright3(i):rend)];
	middle(i) = median(slice);
   noise(i) = median(abs(slice-middle(i)));
   noise(i) = median(abs(diff(newspec(lend:rend))));
end
signal = newspec(y3)-middle;

sn = max(0, signal./noise);

peaks = y3(sn > snminimum);


