function [newspec, keep] = peakRemoval(w, y, yleft, yright, mono)
% NEWSPEC = peakRemoval(spectrum, y, left, right, mono)
%
% Inputs are a spectrum, and a list of peaks (y) with their
% left (yleft) and right (yright) boundaries. There is another
% optional argument (mono). If mono is greater than zero, then
% we believe the ultimate baseline should be monotonically
% decreasing. The default value of mono = 0, which does not
% enfore monotonicity. The output is a new spectrum of the
% same length with the peaks removed and replaced by straight
% lines across the base.
%
% Part of the SPF, SPDBC package described in Clinical Chemistry 2003; 49:1615-1623.
%
% Original version by Kevin Coombes
% $Revision: 2.2 $
% Last modified by $Author: krc $ on $Date: 2003/03/31 19:01:58 $.

if nargin < 5,
   mono = 0;
end

% The problem is what happens if you have several real peaks
% close together. If one peak doesn't come all the way down to
% baseline, you get big jumps in the "peak-removed" spectrum.
% We can start chopping those out, but have to adjust the
% left and right boundaries.

magic = 3;	% magic number of multiples of the median slope
slopes = (w(yright)-w(yleft))./(yright-yleft);
m = magic*median(abs(slopes));
counters = 1:length(slopes);
if mono > 0,
   startup = min(counters(abs(slopes)<m));	% initial slide
else
   temp = 1:length(w);
   screwy = max(temp(w==max(w)));
   startup = min(counters(y > screwy));	% end of saturation
end
m = magic*median(abs(slopes(counters>startup)));	% ignore beginning
selector = ~(counters>startup & abs(slopes) > m);

% only apply this to the peaks with slopes at most 3 times the median
% however, we can have large slopes at the beginning of the spectrum
% in the matrix noise region.

% now we move the left and right boundaries accordingly
z  = y;
zr = yright;
zl = yleft;
for i = 2:length(y)-1,
   if selector(i) == 0,	% forget this peak
      if w(yright(i-1)) > w(yright(i)),
         zr(i-1) = zr(i);
      end
      if w(yleft(i+1)) > w(yleft(i)),
         zl(i+1) = zl(i);
      end
   end
end

% use the selector
z  = z(selector);
zl = zl(selector);
zr = zr(selector);

% finally, remove the peaks
newspec = w;
for i = 1:length(z),
   yr = zr(i);
  	yl = zl(i);
  	slope = (w(yr) - w(yl))/(yr-yl);
   newspec(yl:yr) = slope*((yl:yr) - yl) + w(yl);
end

% still not done... have to make sure we don't get negative values.
newspec = min(newspec, w);
